<?php
/*  
 * Analysis Console for Incident Databases (ACID)
 *
 * Author: Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 *
 * Copyright (C) 2000 Carnegie Mellon University
 * (see the file 'acid_main.php' for license details)
 *
 * Purpose:   
 *
 */


function createDBIndex($db, $table, $field, $index_name)
{
   $sql = 'CREATE INDEX '.$index_name.' ON '.$table.' ('.$field.')';

   $db->acidExecute($sql, -1, -1, false);
   if ( $db->acidErrorMessage() != "" )
      ErrorMessage("Unable to CREATE INDEX for '".$field."' : ".$db->acidErrorMessage());
   else
      ErrorMessage("Successfully created INDEX for '".$field."'");
}

function verify_db($db, $alert_dbname, $alert_host)
{
  $msg = '<B>The underlying database '.$alert_dbname.'@'.$alert_host.' apears to be invalid</B>';

  $sql = "SELECT ip_src FROM iphdr";
  $result = $db->acidExecute($sql, 0, 1, false);
  if ( $db->acidErrorMessage() != "" )
     return $msg.'.  <P>It is an older version.  Only alert databases created by Snort 1.7-beta0 
                  or later are supported'; 
 
  if ( !$db->acidTableExists("acid_ag") )
     return $msg.'.  <P>The database version is valid, but the ACID DB structure (table: acid_ag)
                     is not present. Use the <A HREF="acid_db_setup.php">Setup page</A> to configure
                     and optimize the DB.';

  if ( !$db->acidTableExists("acid_ag_alert") )
     return $msg.'.  <P>The database version is valid, but the ACID DB structure (table: acid_ag_alert)
                     is not present.  Use the <A HREF="acid_db_setup.php">Setup page</A> to configure
                     and optimize the DB.';
  
  return "";
}

/* ******************* DB Query Routines ************************************ */
function EventsByAddr($db, $i, $ip_src0, $ip_src1, $ip_src2, $ip_src3)
{
   $result = $db->acidExecute("SELECT signature FROM event INNER JOIN iphdr ON ".
             "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
             "(ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
             "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')");

   while ( $myrow = $result->acidFetchRow() ) 
      $sig[] = $myrow[0];

   $result->acidFreeRows();

   return $sig[$i];
}

function EventCntByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3)
{
   $result = $db->acidExecute("SELECT count(ip_src0) FROM iphdr WHERE ".
                  "(ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')");

   $myrow = $result->acidFetchRow();
   $event_cnt = $myrow[0];
   $result->acidFreeRows();

   return $event_cnt;
}

function UniqueEventsByAddr($db, $i, $ip_src0, $ip_src1, $ip_src2, $ip_src3)
{
     $result = $db->acidExecute("SELECT DISTINCT signature FROM event INNER JOIN iphdr ON ".
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "(ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')");

   while ($myrow = $result->acidFetchRow())
      $sig[] = $myrow[0];

   $result->acidFreeRow();

   return $sig[$i];
}

function UniqueEventCntByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3)
{
     $result = $db->acidExecute("SELECT DISTINCT signature FROM event INNER JOIN iphdr ON ".
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "(ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')");

   while ($myrow = $result->acidFetchRow())
      $sig[] = $myrow[0];

   $result->acidFreeRows();

   return $sig;
}

function UniqueEventTotalsByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3, $current_event)
{
   $result = $db->acidExecute("SELECT count(signature) FROM event INNER JOIN iphdr ON ". 
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "((ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')) ".
                  "AND signature='$current_event'"); 

   $myrow = $result->acidFetchRow();
   $tmp = $myrow[0];
   
   $result->acidFreeRows();
   return $tmp;
}

function UniqueSensorCntByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3, $current_event)
{
   $result = $db->acidExecute("SELECT DISTINCT event.sid FROM event INNER JOIN iphdr ON ".
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "((ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')) ".
                  "AND signature='$current_event'");

   while ($myrow = $result->acidFetchRow())
      $sid[] = $myrow[0];

   $count = count($sid);
   $result->acidFreeRows();

   return $count;
}

function StartTimeForUniqueEventByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3, $current_event)
{
   $result = $db->acidExecute("SELECT min(timestamp) FROM event INNER JOIN iphdr ON ".
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "((ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')) ".
                  "AND signature = '$current_event';");
   $myrow = $result->acidFetchRow();
   $start_time = $myrow[0];

   $result->acidFreeRows();
   return $start_time;
}

function StopTimeForUniqueEventByAddr($db, $ip_src0, $ip_src1, $ip_src2, $ip_src3, $current_event)
{
   $result = $db->acidExecute("SELECT max(timestamp) FROM event INNER JOIN iphdr ON ".
                  "event.cid=iphdr.cid AND event.sid=iphdr.sid WHERE ".
                  "((ip_src0='$ip_src0' AND ip_src1='$ip_src1' AND ip_src2='$ip_src2' AND ip_src3='$ip_src3') OR ".
                  "(ip_dst0='$ip_src0' AND ip_dst1='$ip_src1' AND ip_dst2='$ip_src2' AND ip_dst3='$ip_src3')) ".
                  "AND signature = '$current_event';");

   $myrow = $result->acidFetchRow();
   $stop_time = $myrow[0];

   $result->acidFreeRows();
   return $stop_time;
}

?>






















